#  Copyright (C) 2020 Jakub Smetana <jakub/AT/smetana/DOT/ml>
#  =========
#  SPDX-License-Identifier: MPL-2.0
#  ---------
#  This Source Code Form is subject to the terms of the Mozilla Public
#  License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
#  =========

from loguru import logger

_VERSION_KEY = "DB_META:version"
_MAX_TRIES = 1
CURRENT_VERSION = 1


class MigrateError(Exception):
    pass


class RedisMigrate(object):
    def __init__(self, redis_connection):
        self.redis = redis_connection

    async def start(self):
        db_version = await self.redis.get(_VERSION_KEY)
        logger.info(f"your db scheme version = {db_version}; latest db scheme version = {CURRENT_VERSION}")
        if db_version is None:
            await self.redis.flushdb()
            await self.redis.set(_VERSION_KEY, CURRENT_VERSION)
            return
        if int(db_version) < CURRENT_VERSION:
            tries = 0
            while int(await self.redis.get(_VERSION_KEY)) != CURRENT_VERSION:
                tries += 1
                if tries > _MAX_TRIES:
                    raise MigrateError
                await self._migrate(db_version)

    async def _migrate(self, from_version):
        pass



#  Copyright (C) 2020 Jakub Smetana <jakub/AT/smetana/DOT/ml>
#  =========
#  SPDX-License-Identifier: MPL-2.0
#  ---------
#  This Source Code Form is subject to the terms of the Mozilla Public
#  License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
#  =========

from mcdbot.mcdbot import global_config
from mcdbot.redis_migrate import RedisMigrate
import aioredis
# from enum import Enum
from loguru import logger
from discord import Member, User
from typing import Union


# class UserStatus(Enum):
#     OK = 0
#     LOCKED = 1


# class McStatus(Enum):
#     ONLINE = 0
#     OFFLINE = 1


class Redis(object):
    def __init__(self):
        self.started = False
        self._redis = None

    async def start(self):
        if not self.started:
            try:
                self._redis = await aioredis.create_redis_pool(global_config.redis_url,
                                                               encoding='utf-8')
                await RedisMigrate(self._redis).start()
            except Exception as e:
                self.started = False
                logger.exception("Failed to start the Redis connector!", e)
                raise ConnectionError
            else:
                self.started = True

    async def stop(self):
        if self.started:
            self.started = False
            self._redis.close()
            await self._redis.wait_closed()

    # DB KEYS DEFINITION ###
    @staticmethod
    def _key_user(uid: int):
        return f"user:{uid}"

    @staticmethod
    def _key_player(player: str):
        return f"player:{player.lower()}"

    # DATA FUNCTIONS ###
    async def create_player(self, player: str, user: Union[Member, User]):
        tr = self._redis.multi_exec()
        tr.set(self._key_user(user.id), player)
        tr.set(self._key_player(player), user.id)

        return await tr.execute()

    async def get_player_by_user(self, user: Union[Member, User]):
        return await self._redis.get(self._key_user(user.id))

    async def has_user_player(self, user: Union[Member, User]):
        return await self._redis.exists(self._key_user(user.id))

    async def player_exists(self, player: str):
        return await self._redis.exists(self._key_player(player))

    async def del_player(self, player: str, user: Union[Member, User]):
        tr = self._redis.multi_exec()
        tr.delete(self._key_player(player))
        tr.delete(self._key_user(user.id))

        return await tr.execute()



#  Copyright (C) 2020 Jakub Smetana <jakub/AT/smetana/DOT/ml>
#  =========
#  SPDX-License-Identifier: MPL-2.0
#  ---------
#  This Source Code Form is subject to the terms of the Mozilla Public
#  License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
#  =========

from mcdbot.errors.http_like_errors import ForbiddenError
from mcdbot.errors.object_errors import McPlayerAlreadyRegisteredError
from mcdbot.helpers import check_user
from mcdbot.mcdbot import Mcdbot

from loguru import logger


async def unregister(main: Mcdbot, msg, context, player: str):
    if check_user(main, msg.author):
        db_player = await main.redis.get_player_by_user(msg.author)
        if db_player == player:
            await main.rcon.authme_unregister(player)
            await main.redis.del_player(player, msg.author)
            logger.info(f"Unregistered {player} belonging to {msg.author.id}.")
        else:
            raise McPlayerAlreadyRegisteredError(db_player)
    else:
        raise ForbiddenError



#  Copyright (C) 2020 Jakub Smetana <jakub/AT/smetana/DOT/ml>
#  =========
#  SPDX-License-Identifier: MPL-2.0
#  ---------
#  This Source Code Form is subject to the terms of the Mozilla Public
#  License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
#  =========

from .mcdbot_error import McdbotError


class McPasswordInvalidUnderPolicyError(McdbotError):
    from mcdbot.helpers import BANNED_PASSWORDS
    NAME = "The Minecraft password is invalid under current password policy! 6 <= length <= 30; only a-zA-Z0-9" \
           "!\"#$%&'()*+,-./:;<=>?@[\\]^_`{|}~ ; " \
           f"must not be one of these: {BANNED_PASSWORDS}"



#  Copyright (C) 2020 Jakub Smetana <jakub/AT/smetana/DOT/ml>
#  =========
#  SPDX-License-Identifier: MPL-2.0
#  ---------
#  This Source Code Form is subject to the terms of the Mozilla Public
#  License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
#  =========

from .mcdbot_error import McdbotError


class ForbiddenError(McdbotError):
    NAME = "403 Forbidden - You probably don't have the required privileges to do the asked action."


class ConflictError(McdbotError):
    NAME = "409 Conflict"


class InternalServerError(McdbotError):
    NAME = "500 Internal Server Error - Please contact the administrator."
